//
// Created by zoe on 12/19/22.
//

#include "TimeManager.h"


TimeManager::TimeManager(TinyGsmSim7000 &modem) : modem(modem) {

}

void TimeManager::syncNTP(const std::string &ntpServer) {
    esp_log_write(ESP_LOG_DEBUG, TAG, "NTP Server Syncing...\n");

    auto error = modem.NTPServerSync(ntpServer.c_str(), timeZone);

    /**
      According to TinGsmNTP.tpp, the error codes are:
      case 1: "Network time synchronization is successful";
      case 61: "Network error";
      case 62: "DNS resolution error";
      case 63: "Connection error";
      case 64: "Service response error";
      case 65: "Service response timeout";
      default: "Unknown error: " + String(error);
     */
    if (error != 1) {
        throw NTPException(modem.ShowNTPError(error).c_str());
    } else {
        esp_log_write(ESP_LOG_DEBUG, TAG, "NTP: %s", modem.ShowNTPError(error).c_str());
    }
}

time_t TimeManager::timeToUnixEpochSeconds(const std::string &time, const std::string &format) {
    //	22/10/27,10:16:20+00
    struct tm tm {};
    time_t dateInEpoch = 0;

    std::stringstream stringStream(time);

    try {
        stringStream >> std::get_time(&tm, format.c_str());
    } catch (std::exception &e) {
        throw StringToTimeConversionException(("Error converting time to epoch %s", e.what()));
    }

    return dateInEpoch;
}

void TimeManager::writeModemTimeToRTC() {
    auto gsmDateTimeString = modem.getGSMDateTime(DATE_FULL);
    esp_log_write(ESP_LOG_DEBUG, TAG, "GSM DateTime: %s\n", gsmDateTimeString.c_str());
    time_t time = timeToUnixEpochSeconds(gsmDateTimeString.c_str());

    rtc.setTime(time);
    esp_log_write(ESP_LOG_INFO, TAG, "Time set to EPOCH: %s\n",
                  String(rtc.getEpoch()).c_str());
}
